==> 0.19.1.zip: koa-lite/docs/koa-vs-express.md <==
THIS DOCUMENT IS IN PROGRESS. THIS PARAGRAPH SHALL BE REMOVED WHEN THIS DOCUMENT IS DONE.

# Koa vs Express

  Philosophically, Koa aims to "fix and replace node", whereas Express "augments node". 
  Koa uses co to rid apps of callback hell and simplify error handling.
  It exposes its own `this.request` and `this.response` objects instead of node's `req` and `res` objects.
  
  Express, on the other hand, augments node's `req` and `res` objects with additional properties and methods 
  and includes many other "framework" features, such as routing and templating, which Koa does not.
  
  Thus, Koa can be viewed as an abstraction of node.js's `http` modules, where as Express is an application framework for node.js.

| Feature           | Koa | Express | Connect |
|------------------:|-----|---------|---------|
| Middleware Kernel | ✓   | ✓       | ✓       |
| Routing           |     | ✓       |         |
| Templating        |     | ✓       |         |
| Sending Files     |     | ✓       |         |
| JSONP             |     | ✓       |         |


  Thus, if you'd like to be closer to node.js and traditional node.js-style coding, you probably want to stick to Connect/Express or similar frameworks. 
  If you want to dive into the land of generators, use Koa.

  As result of this different philosophy is that traditional node.js "middleware", i.e. functions of the form `(req, res, next)`, are incompatible with Koa. Your application will essentially have to be rewritten from the ground, up.

## Does Koa replace Express?

  It's more like Connect, but a lot of the Express goodies
  were moved to the middleware level in Koa to help form
  a stronger foundation. This makes middleware more enjoyable
  and less error-prone to write, for the entire stack, not
  just the end application code.

  Typically many middleware would
  re-implement similar features, or even worse incorrectly implement them,
  when features like signed cookie secrets among others are typically application-specific,
  not middleware specific.

## Does Koa replace Connect?

  No, just a different take on similar functionality
  now that generators allow us to write code with less
  callbacks. Connect is equally capable, and some may still prefer it,
  it's up to what you prefer.

## Why isn't Koa just Express 4.0?

  Koa is a pretty large departure from what people know about Express,
  the design is fundamentally much different, so the migration from
  Express 3.0 to this Express 4.0 would effectively mean rewriting
  the entire application, so we thought it would be more appropriate
  to create a new library.

## Is Koa faster than Express?

Koa and Express have very similar benchmark results, both of which are more than enough for most applications. This is especially true for applications with high traffic running multiple instances, because you are scaling horizontally right?

The following results were produced on a Retina Macbook Pro, showing only the cost of running noop middleware.

| no. middleware | Koa     | Express |
| --------------:| ------- | ------- |
|              1 | 8849.36 | 7805.19 |
|              5 | 8685.74 | 7707.15 |
|             10 | 8511.08 | 7475.08 |
|             15 | 8456.86 | 7261.73 |
|             20 | 8211.93 | 7012.07 |
|             30 | 8102.98 | 6672.76 |
|             50 | 7617.67 | 6255.92 |
|            100 | 6899.45 | 5000.98 |

## How is Koa different than Connect/Express?

### Generated-based control flow

  Thanks to co.

  No callback hell.

  Better error handling through try/catch.

  No need for domains.

### Koa is barebones

  Unlike both Connect and Express, Koa does not include any middleware.

  Unlike Express, routing is not provided.

  Unlike Express, many convenience utilities are not provided. For example, sending files.

  Koa is more modular.

### Koa relies less on middleware

  For example, instead of a "body parsing" middleware, you would instead use a body parsing function.

### Koa abstracts node's request/response

  Less hackery.
  
  Better user experience.

  Proper stream handling.

==> 2.0.1.zip: koa-lite/docs/koa-vs-express.md <==
THIS DOCUMENT IS IN PROGRESS. THIS PARAGRAPH SHALL BE REMOVED WHEN THIS DOCUMENT IS DONE.

# Koa vs Express

  Philosophically, Koa aims to "fix and replace node", whereas Express "augments node".
  Koa uses promises and async functions to rid apps of callback hell and simplify error handling.
  It exposes its own `ctx.request` and `ctx.response` objects instead of node's `req` and `res` objects.

  Express, on the other hand, augments node's `req` and `res` objects with additional properties and methods
  and includes many other "framework" features, such as routing and templating, which Koa does not.

  Thus, Koa can be viewed as an abstraction of node.js's `http` modules, where as Express is an application framework for node.js.

| Feature           | Koa | Express | Connect |
|------------------:|-----|---------|---------|
| Middleware Kernel | ✓   | ✓       | ✓       |
| Routing           |     | ✓       |         |
| Templating        |     | ✓       |         |
| Sending Files     |     | ✓       |         |
| JSONP             |     | ✓       |         |


  Thus, if you'd like to be closer to node.js and traditional node.js-style coding, you probably want to stick to Connect/Express or similar frameworks.
  If you want to get rid of callbacks, use Koa.

  As result of this different philosophy is that traditional node.js "middleware", i.e. functions of the form `(req, res, next)`, are incompatible with Koa. Your application will essentially have to be rewritten from the ground, up.

## Does Koa replace Express?

  It's more like Connect, but a lot of the Express goodies
  were moved to the middleware level in Koa to help form
  a stronger foundation. This makes middleware more enjoyable
  and less error-prone to write, for the entire stack, not
  just the end application code.

  Typically many middleware would
  re-implement similar features, or even worse incorrectly implement them,
  when features like signed cookie secrets among others are typically application-specific,
  not middleware specific.

## Does Koa replace Connect?

  No, just a different take on similar functionality
  now that generators allow us to write code with less
  callbacks. Connect is equally capable, and some may still prefer it,
  it's up to what you prefer.

## Why isn't Koa just Express 4.0?

  Koa is a pretty large departure from what people know about Express,
  the design is fundamentally much different, so the migration from
  Express 3.0 to this Express 4.0 would effectively mean rewriting
  the entire application, so we thought it would be more appropriate
  to create a new library.

## How is Koa different than Connect/Express?

### Generated-based control flow

  No callback hell.

  Better error handling through try/catch.

  No need for domains.

### Koa is barebones

  Unlike both Connect and Express, Koa does not include any middleware.

  Unlike Express, routing is not provided.

  Unlike Express, many convenience utilities are not provided. For example, sending files.

  Koa is more modular.

### Koa relies less on middleware

  For example, instead of a "body parsing" middleware, you would instead use a body parsing function.

### Koa abstracts node's request/response

  Less hackery.

  Better user experience.

  Proper stream handling.
